#!/usr/bin/env python3
import mysql.connector
import collectd
import json

timestamp = 0
interval = 0 # stored interval (5 minutes)

def log_verbose(msg):
    collectd.info('{} plugin [verbose]: {}'.format('PMDB', msg))

def configure_callback(conf):
    log_verbose('Configured with interval={}'.format(60.0))

def write_callback(values):
    global timestamp, interval
    value_dict = {}
    if values.type: value_dict.update({'type': values.type})
    if values.type_instance: value_dict.update({'type_instance': values.type_instance})
    if values.plugin: value_dict.update({'plugin': values.plugin})
    if values.plugin_instance: value_dict.update({'plugin_instance': values.plugin_instance})
    if values.host: value_dict.update({'host': values.host})
    if values.time: value_dict.update({'time': values.time})
    if values.interval: value_dict.update({'interval': values.interval})
    if values.values: value_dict.update({'values': values.values})
    if values.meta: value_dict.update({'meta': values.meta})

    if abs(int(values.time) - timestamp) > 290:
        if timestamp != 0:
            interval = interval + 1
        timestamp = values.time
    mydb = mysql.connector.connect(host='localhost', user="root", passwd="root", database="collectd")
    cursor = mydb.cursor()
    sql = "INSERT INTO pmstats (interval_id, time,data) values(%s, %s, '%s')"%(interval, int(values.time), mydb.converter.escape(json.dumps(value_dict)))
    cursor.execute(sql)
    sql = "UPDATE state_data SET interval_id = %s WHERE row_id = 1"%interval
    cursor.execute(sql)
    mydb.commit()
    if interval%12 == 0:
        sql = "DELETE FROM pmstats WHERE time < unix_timestamp()-(24*60*60)"
        cursor.execute(sql)
        mydb.commit()
    mydb.close()

def init_callback():
    global interval
    mydb = mysql.connector.connect(host='localhost', user="root", passwd="root")
    cursor = mydb.cursor()
    sql = 'CREATE DATABASE IF NOT EXISTS collectd'
    cursor.execute(sql)
    sql = 'USE collectd' # add fields
    cursor.execute(sql)
    sql = 'CREATE TABLE IF NOT EXISTS pmstats (interval_id int, time int, data varchar(400))' # add fields
    cursor.execute(sql)
    mydb.commit()
    sql = "DELETE FROM pmstats WHERE time < unix_timestamp()-(24*60*60)"
    cursor.execute(sql)
    mydb.commit()
    sql = 'CREATE TABLE IF NOT EXISTS state_data (row_id int,interval_id int, PRIMARY KEY (`row_id`))'
    cursor.execute(sql)
    sql = "INSERT IGNORE INTO state_data (row_id,interval_id) values(1,1)"
    cursor.execute(sql)
    mydb.commit()
    sql = "SELECT * FROM state_data"
    cursor.execute(sql)
    records = cursor.fetchall()
    for row in records:
        interval = row[1] + 1
        break
    mydb.commit()
    mydb.close()

    collectd.register_config(configure_callback)
    collectd.register_write(write_callback)

collectd.register_init(init_callback)
