#!/bin/python
import collectd
import pyroute2.netlink.nl80211 as nl
import pyroute2.iwutil as iwu
import socket
PLUGIN_NAME = 'hostapd plugin'
INTERFACE = ""
INTERFACEINDEX = 0
CLIENTS = []
CLIENTS_DB = []

# Plugin initialized
def init():
    collectd.info("hostapd plugin : loaded")

# Parse and configure plugin
def config_function(config):
    # Global variables storing conf and shared ressources
    global INTERFACE
    global CLIENTS
    global INTERFACEINDEX
    for node in config.children:
        key = node.key.lower()
        value = node.values[0]
        # Save Wi-Fi interface name
        if key == 'interface':
            INTERFACE = value
            collectd.info("Collecting stats for interface %s" % value)
        # Save a client MAC address
        elif key == 'client':
            CLIENTS.append(value)
        else:
            collectd.info("Unknown configuration key %s" % key)
    # No clients configured ? Store everything
    if len(CLIENTS) == 0:
        collectd.info("Collecting detailed stats for all clients")
    else:
        str_clients = ""
        for client in CLIENTS:
            str_clients = str_clients + " " +client
        collectd.info("Collecting stats for clients%s" % str_clients)

    # No Wi-FI interface configured
    if INTERFACE == "":
        collectd.error("No interface set in configuration")
        return
    else:
        # Get the interface index from the interface name
        msg = nl.nl80211cmd()
        msg['cmd'] = nl.NL80211_NAMES['NL80211_CMD_GET_INTERFACE']
        msg['attrs'] = [['NL80211_ATTR_IFNAME', INTERFACE]]
        result = []
        conn = iwu.IW()
        result = conn.nlm_request(msg, msg_type=conn.prid, msg_flags=iwu.NLM_F_REQUEST | iwu.NLM_F_DUMP)
        if len(result) == 0:
            collectd.error("Unknown network interface !")
            conn.close()
            return
        else:
            for iface in result:
                for attr in iface['attrs']:
                    if 'NL80211_ATTR_IFINDEX' in attr:
                        ifindex = attr[1]
                    if 'NL80211_ATTR_IFNAME' in attr:
                        ifname = attr[1]
            if INTERFACE in ifname:
                INTERFACEINDEX = ifindex
        conn.close()

# Actually send the data
def send_notification(message):
    global PLUGIN_NAME
    try:
        with open('/config/collectd-hostname', 'r') as f:
            host = f.readline().strip()
    except:
        host=socket.gethostname()
    notif = collectd.Notification(plugin=PLUGIN_NAME,
                                      type="absolute",
                                      severity=4,
                                      message=message,
                                      host=host)
    notif.dispatch()
        
def send_disconnect(station):
    message = "Station (%s) disconnected"%station
    send_notification(message)

def send_connect(station):
    message = "Station (%s) connected"%station
    send_notification(message)

def terminate_function():
    collectd.info("Terminating Hostapd plugin")

# Collect and send data every n seconds
def read_function():
    results = []
    global INTERFACEINDEX

    try:
        conn = iwu.IW()
        results = conn.get_stations(INTERFACEINDEX)

        stations_db = []
        if len(results) >= 0:
            for sta in results:
                for attr in sta['attrs']:
                    if 'NL80211_ATTR_MAC' in attr:
                        sta_mac = attr[1]
                        stations_db.append(sta_mac)
                        break
            s = set(stations_db)
            rm_stations = [x for x in CLIENTS_DB if x not in s]
            s = set(CLIENTS_DB)
            add_stations = [x for x in stations_db if x not in s]
            for i in rm_stations:
                send_disconnect(i)
                CLIENTS_DB.remove(i)
            for i in add_stations:
                send_connect(i)
            CLIENTS_DB.extend(add_stations)

        # Clean a few values to avoid memory leak
        del(results)
        conn.close()
    except:
        pass

# Register various functions called during the various stages of the daemon
collectd.register_init(init)
collectd.register_config(config_function)
collectd.register_read(read_function)
collectd.register_shutdown(terminate_function)
