#!/usr/bin/env python3
# -*- coding: utf-8 -*-
import json
import os
import subprocess
import sys
import syslog

import mysql.connector

if __name__ != '__main__':
    import collectd

# Save last hypervisor status
f_path = "/run/hv_status/"
last_hv_status = f_path + "last_hv_status.json"

hv_status_template = "{\"hypervisors\": {}}"


def get_hv_status(hv):
    data = None
    if not (os.path.exists(last_hv_status)):
        try:
            os.mkdir(f_path)
        except Exception as e:
            pass
        hv_status_f = open(last_hv_status, 'w')
        data = json.loads(hv_status_template)
        json.dump(data, hv_status_f)
        hv_status_f.close()
        os.chmod(last_hv_status, 0o777)
    else:
        with open(last_hv_status, "r", encoding='utf-8') as json_file:
            data = json.load(json_file)
            json_file.close()

    if data and hv in data['hypervisors']:
        return data['hypervisors'][hv]['last-status']

    return "unknown"


def update_hv_status(hv, status):
    with open(last_hv_status, "r+", encoding='utf-8') as json_file:
        data = json.load(json_file)

        data['hypervisors'][hv]['last-status'] = str(status)

        json_file.seek(0)
        json.dump(data, json_file)
        json_file.truncate()
        json_file.close()


def add_hv_status(hv, status):
    with open(last_hv_status, "r+", encoding='utf-8') as json_file:
        data = json.load(json_file)

        # Add new hypervisor in status file
        data['hypervisors'].update({hv: {'last-status': status}})

        json_file.seek(0)
        json.dump(data, json_file)
        json_file.truncate()
        json_file.close()


def hypervisor_status():
    val = collectd.Notification(type='absolute')
    val.plugin = 'hypervisor_status'
    # DB connector timeout (31536000 minutes is equal to 1 year)
    # db.query('SET GLOBAL connect_timeout=31536000')
    try:
        db = mysql.connector.connect(host="localhost", user="root", passwd="root", database="nova")
    except mysql.connector.Error as err:
        if err.errno == errorcode.ER_ACCESS_DENIED_ERROR:
            syslog.syslog(syslog.LOG_ERR, "Something is wrong with your user name or password!")
            sys.exit(1)
        elif err.errno == errorcode.ER_BAD_DB_ERROR:
            syslog.syslog(syslog.LOG_ERR, "Database does not exist!")
            sys.exit(1)
        else:
            syslog.syslog(syslog.LOG_ERR, str(err))
            sys.exit(1)

    # Get buffered cursor
    cursor_down = db.cursor(buffered=True)
    cursor_up = db.cursor(buffered=True)

    # Query to get offline hypervisors in last 11 minutes
    cursor_down.execute("SELECT id, host, created_at, last_seen_up, UTC_TIME() FROM services WHERE topic = 'compute' AND last_seen_up <= DATE_SUB(UTC_TIME(), INTERVAL 660 SECOND);")
    for hypervisor in cursor_down:
        if hypervisor[0]:
            last_status = get_hv_status(hypervisor[1])
            # Message to be send by collectd
            msg = 'Hypervisor state of \"{}\" with ID \"{}\" has been changed to \"DOWN\"'.format(hypervisor[1],
                                                                                                  hypervisor[0])

            if last_status == 'unknown':
                add_hv_status(hypervisor[1], 'DOWN')
                val.dispatch(severity=1, message=msg)
            elif last_status == 'UP':
                update_hv_status(hypervisor[1], 'DOWN')
                val.dispatch(severity=1, message=msg)

    # Query to get online hypervisors in last 11 minutes
    cursor_up.execute(
        "SELECT id, host, created_at, last_seen_up, UTC_TIME() FROM services WHERE topic = 'compute' AND last_seen_up >= DATE_SUB(UTC_TIME(), INTERVAL 660 SECOND);")
    for hypervisor in cursor_up:
        if hypervisor[0]:
            last_status = get_hv_status(hypervisor[1])
            # Message to be send by collectd
            msg = 'Hypervisor state of \"{}\" with ID \"{}\" has been changed to \"UP\"'.format(hypervisor[1],
                                                                                                hypervisor[0])

            if last_status == 'unknown':
                add_hv_status(hypervisor[1], 'UP')
                val.dispatch(severity=1, message=msg)
            elif last_status == 'DOWN':
                update_hv_status(hypervisor[1], 'UP')
                val.dispatch(severity=1, message=msg)

    # Close DB connection
    db.close()


if __name__ == '__main__':
    print("Hypervisor status plugin")
else:
    # Check for CBOX mode is active
    try:
        state = subprocess.check_output(['systemctl', 'is-active', 'nova-api'])
        if state.decode().rstrip() == 'active':
            collectd.register_read(hypervisor_status)
    except subprocess.CalledProcessError as e:
        pass
