#!/usr/bin/python3

# Sample python plugin
# For more information see https://collectd.org/documentation/manpages/collectd-python.5.shtml

import sys
import time
from math import pi, sin
import os
import socket
import threading
import collectd
import logging
import json
import subprocess
import requests

#import vppapi_if.vppapi_if as vppif
#from pyroute2 import IPRSocket
from systemd import journal

PLUGIN_NAME = 'k8s_stats'
RATE_INTERVAL = 880 # 15 minutes
MB = 1024*1024

class RateCounters(object):
    pod_ifname= ""
    last_rx_bytes_count = 0
    last_tx_bytes_count = 0
    last_rx_packets_count = 0
    last_tx_packets_count = 0
    last_time = time.time() #time of collecting counters above
    def __init__(self, name, rx_b, tx_b, rx_p, tx_p, time):
        self.pod_ifname = name
        self.last_rx_bytes_count = rx_b
        self.last_tx_bytes_count = tx_b
        self.last_rx_packets_count = rx_p
        self.last_tx_packets_count = tx_p
        self.last_time = time

IfDB = list()

class vm_stats(object):
    pod_vmname= ""
    last_cpu_value = 0
    last_time = time.time() #time of collecting stats above
    def __init__(self, name, cpu, time):
        self.pod_vmname = name
        self.last_cpu_value = cpu
        self.last_time = time

IfDB_vm_stats = list()

def print_ifdb():
    log("--------------  if db list, number of entries: " + str(len(IfDB)) + " -----------")
    for entry in range(len(IfDB)):
        collectd.info("%s: %s %s %s %s %s %s" % (PLUGIN_NAME, IfDB[entry].pod_ifname, IfDB[entry].last_time, \
        IfDB[entry].last_rx_bytes_count, IfDB[entry].last_tx_bytes_count, \
        IfDB[entry].last_rx_packets_count, IfDB[entry].last_tx_packets_count))

def print_vm_stats_ifdb():
    log("--------------  vm_stats list, number of entries: " + str(len(IfDB_vm_stats)) + " -----------")
    for entry in range(len(IfDB_vm_stats)):
        collectd.info("%s: %s %s %s" % (PLUGIN_NAME, IfDB_vm_stats[entry].pod_vmname, IfDB_vm_stats[entry].last_time, IfDB_vm_stats[entry].last_cpu_value))

def log(param):
    """
    Log messages to either collectd or stdout depending on how it was called.

    :param param: the message
    :return:  None
    """

    if __name__ != '__main__':
        collectd.info("%s: %s" % (PLUGIN_NAME, param))
    else:
        sys.stderr.write("%s\n" % param)


def config(conf):
    """
    This method has been registered as the config callback and is used to parse options from
    given config.  Note that you cannot receive the whole config files this way, only Module blocks
    inside the Python configuration block. Additionally you will only receive blocks where your
    callback identifier matches your plugin.

    In this case Frequency is a float value that will modify the frequency of the sine wave. This
    in conjunction with the polling interval can give you as smooth or blocky a curve as you want.

    :param conf: a Config object
    :return: None
    """

    #for kv in conf.children:
    #    if kv.key == 'Frequency':
    #        global FREQUENCY
    #        FREQUENCY = float(kv.values[0])


def read():
    """
    This method has been registered as the read callback and will be called every polling interval
    to dispatch metrics.  We emit three metrics: one gauge, a sine wave; two counters for the
    number of datapoints and notifications we've seen.

    :return: None
    """
    
    all_pods = subprocess.check_output("export KUBECONFIG=/etc/kubernetes/admin.conf; kubectl get pods -A -o jsonpath=\"{.items[*].metadata.name}\"", shell=True)
    all_nodes = subprocess.check_output("export KUBECONFIG=/etc/kubernetes/admin.conf; kubectl get nodes -A -o jsonpath=\"{.items[*].metadata.name}\"", shell=True)
    for pod_name in all_pods.decode('utf8', errors='ignore').split(' '):
        #log (pod_name)
        #pod_name = "calico-kube-controllers-596854fbbf-pbs6m"
        try:
            ###
            ###container_memory_usage_bytes
            ###container_memory_rss

            tmp = ""
            tags = ""
            prom_metric = "container_memory_usage_bytes"
            get_tags = 'curl --silent --data-urlencode \'query=%s{pod=\"%s\",container=\"\"}\' http://localhost:30000/api/v1/query'%(prom_metric, pod_name)
            tmp = subprocess.check_output(get_tags, shell=True).decode('utf8').strip()

            if tmp != "":
                tags = json.loads(subprocess.check_output('echo \'%s\' | python3 -mjson.tool' %(tmp), shell=True))

            #prom_metric = "container_memory_usage_bytes"
            #get_tags = 'curl --silent --data-urlencode \'query=%s{pod=\"%s\",container=\"\"}\' http://localhost:30000/api/v1/query | python3 -mjson.tool'%(prom_metric, pod_name)
            #tags = json.loads(subprocess.check_output(get_tags, shell=True))

            if tags != "" and tags['data']['result'] != []:
                value = tags['data']['result'][0]['value'][1]
                metric = collectd.Values()
                metric.plugin = PLUGIN_NAME
                metric.plugin_instance = "memory_usage"
                metric.type_instance = pod_name
                #metric.type = "val"
                metric.type = "gauge"
                metric.values = [value]
                metric.dispatch()

            ###
            ### container_cpu_usage_seconds_total
            ### node_namespace_pod_container:container_cpu_usage_seconds_total:sum_irate
            ### rate(container_cpu_usage_seconds_total{container!="POD"}[10m])/(container_spec_cpu_quota / container_spec_cpu_period)
            ###
            #get_tags = 'curl --silent --data-urlencode \'query=container_cpu_usage_seconds_total{pod=\"%s\",container=\"\"}\' http://localhost:30000/api/v1/query | python3 -mjson.tool'%(pod_name)
            #tags = json.loads(subprocess.check_output(get_tags, shell=True))

            #value = tags['data']['result'][0]['value'][1]
            #metric = collectd.Values()
            #metric.plugin = PLUGIN_NAME
            #metric.plugin_instance = "cpu_usage"
            #metric.type_instance = pod_name
            ##metric.type = "val"
            #metric.type = "gauge"
            #metric.values = [value]
            #metric.dispatch()

            ###
            ###container_fs_usage_bytes
            ###
            # vstojkov: Get statistics for FS from /api/v1/nodes/vs-controller/proxy/stats/summary , because of bug below
            # https://github.com/google/cadvisor/issues/2785 (Check for new fixes from community and fix here)
            # Slow search operation

            tmp = ""
            tags = ""
            #get_tags = 'curl --silent  --data-urlencode \'query=container_fs_usage_bytes{pod=\"%s\"}\' http://localhost:30000/api/v1/query '%(pod_name)
            #tmp = subprocess.check_output(get_tags, shell=True).decode('utf8').strip()
            for node in all_nodes.decode('utf8', errors='ignore').split(' '):
                tmp = subprocess.check_output('export KUBECONFIG=/etc/kubernetes/admin.conf; kubectl get --raw \"/api/v1/nodes/%s/proxy/stats/summary\" | python3 -mjson.tool'%(node), shell=True)

                if tmp != "":
                    #tags = json.loads(subprocess.check_output('echo \'%s\' | python3 -mjson.tool' %(tmp), shell=True))
                    tags = json.loads(tmp)

                if tags != "":
                    for pod in tags['pods']:
                        if pod['podRef']['name'] == pod_name:
                            value = pod['containers'][0]['rootfs']['usedBytes']
                            #value = tags['data']['result'][0]['value'][1]
                            metric = collectd.Values()
                            metric.plugin = PLUGIN_NAME
                            metric.plugin_instance = "fs_usage"
                            # type instance is : pod_container
                            #metric.type_instance = pod_name + '_' + tags['data']['result'][0]['metric']['container']
                            metric.type_instance = pod_name + '_' + pod['containers'][0]['name']
                            #metric.type = "val"
                            metric.type = "gauge"
                            metric.values = [value]
                            metric.dispatch()

                            value = pod['ephemeral-storage']['usedBytes']
                            metric = collectd.Values()
                            metric.plugin = PLUGIN_NAME
                            metric.plugin_instance = "fs_usage"
                            metric.type_instance = pod_name + '_POD'
                            #metric.type = "val"
                            metric.type = "gauge"
                            metric.values = [value]
                            metric.dispatch()

        except Exception as e:
            collectd.error("VSLog: k8s stats read function failed, exception: {}".format(e))

    ###
    ### network interface statistics
    ###
    # eth0 for all pods
    #get_tags = 'curl --silent  --data-urlencode \'query=container_network_receive_bytes_total{pod!=\"\"}\' http://localhost:30000/api/v1/query | python3 -mjson.tool'
    #tags = json.loads(subprocess.check_output(get_tags, shell=True))
    #for entry in tags['data']['result']:
    #    if entry['metric']['interface'] == "eth0":
    #        #log(entry['metric']['pod'])
    #        value = entry['value'][1]
    #        metric = collectd.Values()
    #        metric.plugin = PLUGIN_NAME
    #        metric.plugin_instance = entry['metric']['interface']
    #        metric.type_instance = "if_octets_rx" + "-" + entry['metric']['pod']
    #        #metric.type = "val"
    #        metric.type = "counter"
    #        metric.values = [value]
    #        metric.dispatch()

    #eth0
    #get_tags = 'curl --silent  --data-urlencode \'query=container_network_transmit_bytes_total{pod!=\"\",interface=\"eth0\"}\' http://localhost:30000/api/v1/query | python3 -mjson.tool'
    #tags = json.loads(subprocess.check_output(get_tags, shell=True))
    #for entry in tags['data']['result']:
    #    value = entry['value'][1]
    #    metric = collectd.Values()
    #    metric.plugin = PLUGIN_NAME
    #    metric.plugin_instance = entry['metric']['interface']
    #    metric.type_instance = "if_octets_tx" + "-" + entry['metric']['pod']
    #    metric.type = "counter"
    #    metric.values = [value]
    #    metric.dispatch()


    #network stats for custom pod interfaces - rates
    print_ifdb()
    current_time = time.time()
    log(current_time)
    tags_rx_bytes = ""
    tmp = ""
    
    try:
        get_tags_rx_bytes = 'curl --silent --data-urlencode \'query=container_network_receive_bytes_total{namespace=\"default\"}\' http://localhost:30000/api/v1/query'
        tmp = subprocess.check_output(get_tags_rx_bytes, shell=True).decode('utf8').strip()
        if tmp != "":
            tags_rx_bytes = json.loads(subprocess.check_output('echo \'%s\' | python3 -mjson.tool' %(tmp), shell=True))

#            get_tags = 'curl --silent --data-urlencode \'query=%s{pod=\"%s\",container=\"\"}\' http://localhost:30000/api/v1/query | python3 -mjson.tool'%(prom_metric, pod_name)
#            tags = json.loads(subprocess.check_output(get_tags, shell=True))

        #get_tags_rx_bytes = 'curl --silent --data-urlencode \'query=container_network_receive_bytes_total{namespace=\"default\"}\' http://localhost:30000/api/v1/query | python3 -mjson.tool'
        #tags_rx_bytes = json.loads(subprocess.check_output(get_tags_rx_bytes, shell=True))

            get_tags_tx_bytes = 'curl --silent --data-urlencode \'query=container_network_transmit_bytes_total{namespace=\"default\"}\' http://localhost:30000/api/v1/query | python3 -mjson.tool'
            tags_tx_bytes = json.loads(subprocess.check_output(get_tags_tx_bytes, shell=True))

            get_tags_rx_packets = 'curl --silent --data-urlencode \'query=container_network_receive_packets_total{namespace=\"default\"}\' http://localhost:30000/api/v1/query | python3 -mjson.tool'
            tags_rx_packets = json.loads(subprocess.check_output(get_tags_rx_packets, shell=True))

            get_tags_tx_packets = 'curl --silent --data-urlencode \'query=container_network_transmit_packets_total{namespace=\"default\"}\' http://localhost:30000/api/v1/query | python3 -mjson.tool'
            tags_tx_packets = json.loads(subprocess.check_output(get_tags_tx_packets, shell=True))
    except Exception as e:
        collectd.error("k8s stats read function failed (net stats), exception: {}".format(e))

    if tags_rx_bytes == "":
        collectd.error("k8s stats read function failed (no net stats available)")
    else:
        try:
            for entry in tags_rx_bytes['data']['result']:
                if "sit" in entry['metric']['interface']:
                    continue

                found = False
                stats_interval = 0
                # find the entry in our DB for this interface
                for i in range(len(IfDB)):
                    if IfDB[i].pod_ifname == entry['metric']['pod'] + "_" + entry['metric']['interface']:
                        found = True
                        break

                value = entry['value'][1]
                pod_ifname = entry['metric']['pod'] + "_" + entry['metric']['interface']
                if not found:
                    # entry was not found, add it, do not send any rates
                    for entry in tags_tx_bytes['data']['result']:
                        if pod_ifname == entry['metric']['pod'] + "_" + entry['metric']['interface']:
                            value_tx_bytes = entry['value'][1]
                            #log(value_tx_bytes)
                            break
                    for entry in tags_rx_packets['data']['result']:
                        if pod_ifname == entry['metric']['pod'] + "_" + entry['metric']['interface']:
                            value_rx_packets = entry['value'][1]
                            #log(value_rx_packets)
                            break
                    for entry in tags_tx_packets['data']['result']:
                        if pod_ifname == entry['metric']['pod'] + "_" + entry['metric']['interface']:
                            value_tx_packets = entry['value'][1]
                            #log(value_tx_packets)
                            break

                    IfDB.append(RateCounters(pod_ifname, value, value_tx_bytes, value_rx_packets, value_tx_packets, current_time))
                    collectd.info("%s: %s %s %s %d %s %s / %s / %s / %s" % (PLUGIN_NAME, "Adding entry:", pod_ifname, ", time:", int(current_time), " counters: ", value, value_tx_bytes, value_rx_packets, value_tx_packets))
                else:
                    stats_interval = float(current_time) - float(IfDB[i].last_time)
                    if stats_interval >= RATE_INTERVAL:
                        #log("time to get and send rates!!!!!")
                        # it is time for sending rates
                        metric = collectd.Values()
                        metric.plugin = PLUGIN_NAME
                        metric.plugin_instance = pod_ifname
                        metric.type_instance = "if_rx_bytes_rate"
                        metric.type = "gauge"
                        # send the rate
                        rate = (int(value) - int(IfDB[i].last_rx_bytes_count))/(float(stats_interval))
                        metric.values = [rate]
                        if rate >= 0:
                            metric.dispatch()
                        # save current counters in database
                        IfDB[i].last_time = current_time
                        IfDB[i].last_rx_bytes_count = value
                        collectd.info("%s: %s %s %s %d %s %s/%d" % (PLUGIN_NAME, "Updating entry:", pod_ifname, ", time:", int(current_time), " rx byte count/rate:", value, rate))

                        for entry in tags_tx_bytes['data']['result']:
                            if pod_ifname == entry['metric']['pod'] + "_" + entry['metric']['interface']:
                                metric = collectd.Values()
                                metric.plugin = PLUGIN_NAME
                                metric.plugin_instance = pod_ifname
                                metric.type_instance = "if_tx_bytes_rate"
                                metric.type = "gauge"
                                # send the rate
                                value = entry['value'][1]
                                rate = (int(value) - int(IfDB[i].last_tx_bytes_count))/(float(stats_interval))
                                metric.values = [rate]
                                if rate >= 0:
                                    metric.dispatch()
                                # save current counters in database
                                #IfDB[i].last_time = current_time
                                IfDB[i].last_tx_bytes_count = value
                                collectd.info("%s: %s %s %s %d %s %s/%d" % (PLUGIN_NAME, "Updating entry:", pod_ifname, ", time:", int(current_time), " tx byte count/rate:", value, rate))
                                break

                        for entry in tags_rx_packets['data']['result']:
                            if pod_ifname == entry['metric']['pod'] + "_" + entry['metric']['interface']:
                                metric = collectd.Values()
                                metric.plugin = PLUGIN_NAME
                                metric.plugin_instance = pod_ifname
                                metric.type_instance = "if_rx_packets_rate"
                                metric.type = "gauge"
                                # send the rate
                                value = entry['value'][1]
                                rate = (int(value) - int(IfDB[i].last_rx_packets_count))/(float(stats_interval))
                                metric.values = [rate]
                                if rate >= 0:
                                    metric.dispatch()
                                # save current counters in database
                                #IfDB[i].last_time = current_time
                                IfDB[i].last_rx_packets_count = value
                                collectd.info("%s: %s %s %s %d %s %s/%d" % (PLUGIN_NAME, "Updating entry:", pod_ifname, ", time:", int(current_time), " rx packet count/rate:", value, rate))
                                break

                        for entry in tags_tx_packets['data']['result']:
                            if pod_ifname == entry['metric']['pod'] + "_" + entry['metric']['interface']:
                                metric = collectd.Values()
                                metric.plugin = PLUGIN_NAME
                                metric.plugin_instance = pod_ifname
                                metric.type_instance = "if_tx_packets_rate"
                                metric.type = "gauge"
                                # send the rate
                                value = entry['value'][1]
                                rate = (int(value) - int(IfDB[i].last_tx_packets_count))/(float(stats_interval))
                                metric.values = [rate]
                                if rate >= 0:
                                    metric.dispatch()
                                # save current counters in database
                                #IfDB[i].last_time = current_time
                                IfDB[i].last_tx_packets_count = value
                                collectd.info("%s: %s %s %s %d %s %s/%d" % (PLUGIN_NAME, "Updating entry:", pod_ifname, ", time:", int(current_time), " tx packet count/rate:", value, rate))
                                break
        except Exception as e:
            collectd.error("k8s stats read function failed (net rate stats), exception: {}".format(e))

    print_ifdb()

    # clear database - delete stale entries
    for entry in IfDB:
        if (int(current_time) - int(entry.last_time)) > (2 * RATE_INTERVAL):
            collectd.info("%s: %s %s %s %d %s %s" % (PLUGIN_NAME, "Removing entry:", entry.pod_ifname, ", current time:", int(current_time), " entry last update time:", entry.last_time))
            IfDB.remove(entry)

    try:
        tmp = ""
        tags = ""
        get_tags = 'curl --silent --data-urlencode \'query=container_network_receive_errors_total{namespace=\"default\"}\' http://localhost:30000/api/v1/query'
        tmp = subprocess.check_output(get_tags, shell=True).decode('utf8').strip()
        if tmp != "":
            tags = json.loads(subprocess.check_output('echo \'%s\' | python3 -mjson.tool' %(tmp), shell=True))

        if tags != "":
            for entry in tags['data']['result']:
                if not entry['metric']['interface'].startswith("sit"):
                    value = entry['value'][1]
                    metric = collectd.Values()
                    metric.plugin = PLUGIN_NAME
                    metric.plugin_instance = entry['metric']['pod'] + "-" + entry['metric']['interface']
                    metric.type_instance = "if_rx_errors"
                    #metric.type = "val"
                    metric.type = "counter"
                    metric.values = [value]
                    metric.dispatch()

        tmp = ""
        tags = ""
        get_tags = 'curl --silent --data-urlencode \'query=container_network_transmit_errors_total{namespace=\"default\"}\' http://localhost:30000/api/v1/query'
        tmp = subprocess.check_output(get_tags, shell=True).decode('utf8').strip()
        if tmp != "":
            tags = json.loads(subprocess.check_output('echo \'%s\' | python3 -mjson.tool' %(tmp), shell=True))
            
        if tags != "":
            for entry in tags['data']['result']:
                if not entry['metric']['interface'].startswith("sit"):
                    value = entry['value'][1]
                    metric = collectd.Values()
                    metric.plugin = PLUGIN_NAME
                    metric.plugin_instance = entry['metric']['pod'] + "-" + entry['metric']['interface']
                    metric.type_instance = "if_tx_errors"
                    #metric.type = "val"
                    metric.type = "counter"
                    metric.values = [value]
                    metric.dispatch()

        tmp = ""
        tags = ""
        get_tags = 'curl --silent --data-urlencode \'query=container_network_receive_packets_dropped_total{namespace=\"default\"}\' http://localhost:30000/api/v1/query'
        tmp = subprocess.check_output(get_tags, shell=True).decode('utf8').strip()
        if tmp != "":
            tags = json.loads(subprocess.check_output('echo \'%s\' | python3 -mjson.tool' %(tmp), shell=True))

        if tags != "":
            for entry in tags['data']['result']:
                if not entry['metric']['interface'].startswith("sit"):
                    value = entry['value'][1]
                    metric = collectd.Values()
                    metric.plugin = PLUGIN_NAME
                    metric.plugin_instance = entry['metric']['pod'] + "-" + entry['metric']['interface']
                    metric.type_instance = "if_rx_dropped"
                    #metric.type = "val"
                    metric.type = "counter"
                    metric.values = [value]
                    metric.dispatch()

        tmp = ""
        tags = ""
        get_tags = 'curl --silent --data-urlencode \'query=container_network_transmit_packets_dropped_total{namespace=\"default\"}\' http://localhost:30000/api/v1/query'
        tmp = subprocess.check_output(get_tags, shell=True).decode('utf8').strip()
        if tmp != "":
            tags = json.loads(subprocess.check_output('echo \'%s\' | python3 -mjson.tool' %(tmp), shell=True))

        if tags != "":
            for entry in tags['data']['result']:
                if not entry['metric']['interface'].startswith("sit"):
                    value = entry['value'][1]
                    metric = collectd.Values()
                    metric.plugin = PLUGIN_NAME
                    metric.plugin_instance = entry['metric']['pod'] + "-" + entry['metric']['interface']
                    metric.type_instance = "if_tx_dropped"
                    #metric.type = "val"
                    metric.type = "counter"
                    metric.values = [value]
                    metric.dispatch()

    except Exception as e:
        collectd.error("k8s stats read function failed (net errors), exception: {}".format(e))

    ###
    ###cpu usage stats
    ###
    ### container_cpu_usage_seconds_total
    ### node_namespace_pod_container:container_cpu_usage_seconds_total:sum_irate
    ### rate(container_cpu_usage_seconds_total{container!="POD"}[10m])/(container_spec_cpu_quota / container_spec_cpu_period)
    ###

    try:
        tmp = ""
        tags_q = ""
        tags_quota = 'curl --silent --data-urlencode \'query=(container_spec_cpu_quota/container_spec_cpu_period{container!="POD",container!=""})\' http://localhost:30000/api/v1/query'
        tmp = subprocess.check_output(tags_quota, shell=True).decode('utf8').strip()

        if tmp != "":
            tags_q = json.loads(subprocess.check_output('echo \'%s\' | python3 -mjson.tool' %(tmp), shell=True))
        #for entry in tags_q['data']['result']: 
            #log(entry['metric']['pod']+"_"+entry['metric']['container'])
            #log(entry['metric']['name'])
            #log(entry['value'][1])

        tmp = ""
        tags = ""
        get_tags = 'curl --silent --data-urlencode \'query=rate(container_cpu_usage_seconds_total{container!="POD",container!=""}[10m])\' http://localhost:30000/api/v1/query'
        tmp = subprocess.check_output(get_tags, shell=True).decode('utf8').strip()

        if tmp != "":
            tags = json.loads(subprocess.check_output('echo \'%s\' | python3 -mjson.tool' %(tmp), shell=True))

        if tags != "":
            for entry in tags['data']['result']:
                cpu_quota = 1
                for entry_q in tags_q['data']['result']:
                    if entry_q['metric']['name'] == entry['metric']['name']:
                        cpu_quota = entry_q['value'][1]
                        break
                if cpu_quota == 1:
                    continue
                #log(entry['metric']['name'])
                #log(entry['value'][1])
                #log(cpu_quota)
                value = (float(entry['value'][1]))*100/(float(cpu_quota))
                metric = collectd.Values()
                metric.plugin = PLUGIN_NAME
                metric.plugin_instance = "cpu_usage"
                metric.type_instance = entry['metric']['pod'] + "_" + entry['metric']['container']
                #metric.type = "val"
                metric.type = "gauge"
                metric.values = [value]
                metric.dispatch()
    except Exception as e:
        collectd.error("k8s stats read function failed (cpu stats), exception: {}".format(e))

    #################
    # VM statistics #
    #################
    try:
        cmd = "kubectl get vm | grep -v No\ resources\ found  | grep True"
        running_vms = (subprocess.check_output("export KUBECONFIG=/etc/kubernetes/admin.conf; %s" %(cmd), shell=True)).decode('utf8', errors='ignore').split('\n')
        #log(running_vms)
    except Exception as e:
        collectd.error("k8s stats read function failed (vm stats), exception: {}".format(e))

    # clear database - delete stale entries
    removed = False
    for entry in IfDB_vm_stats:
        if (int(current_time) - (int(entry.last_time))/1000) > (2 * RATE_INTERVAL):
            collectd.info("%s: %s %s %s %d %s %s" % (PLUGIN_NAME, "Removing entry (vm stats db):", entry.pod_vmname, ", current time:", int(current_time), " entry last update time:", entry.last_time))
            IfDB_vm_stats.remove(entry)
            removed = True

    #if removed:
        #print_vm_stats_ifdb()
        
    if running_vms == "":
        return

    try:
        all_pods = (subprocess.check_output("export KUBECONFIG=/etc/kubernetes/admin.conf; kubectl get pods -A -o jsonpath=\"{.items[*].metadata.name}\"", shell=True)).decode('utf8', errors='ignore')
    except Exception as e:
        collectd.error("k8s stats read function failed (vm statistics), exception: {}".format(e))

    for vm in running_vms[:-1]:
        vmname = vm.split(' ')[0]
        #log(vmname)
        try:
            vmhost = (subprocess.check_output("export KUBECONFIG=/etc/kubernetes/admin.conf; kubectl get vm %s -o json | /usr/bin/jq '.spec.template.spec.nodeSelector.ucpe'" %(vmname), shell=True)).decode('utf8').strip()
        except:
            vmhost = socket.gethostname()
    
        for pod_name in all_pods.split(' '):
            if vmname in pod_name:
                #log(pod_name)
                break
        # VM memory usage
        #mem_usage = (subprocess.check_output("export KUBECONFIG=/etc/kubernetes/admin.conf; kubectl top pod | grep %s | tr -s ' '| cut -d' ' -f3" %(pod_name), shell=True)).decode('utf8').strip()

        #if mem_usage.isdigit():
        #    mem = mem_usage
        #elif mem_usage.endswith("Mi"):
        #    mem = int(mem_usage.rstrip("Mi")) * MB
        #else:
        #    mem = 0

        mem_usage = (subprocess.check_output("export KUBECONFIG=/etc/kubernetes/admin.conf; kubectl get --raw /api/v1/nodes/%s/proxy/metrics/resource | grep pod_memory_working_set_bytes | grep %s | cut -d' ' -f2" %(vmhost, pod_name), shell=True)).decode('utf8').strip()
        #log(float(mem_usage))
        try:
            mem = int(float(mem_usage))
        except Exception as e:
            collectd.error("k8s stats bad mem_usage {} exception: {}".format(mem_usage,e))
            mem=0
            
        metric = collectd.Values()
        metric.plugin = PLUGIN_NAME
        metric.plugin_instance = "memory_usage"
        metric.type_instance = vmname
        metric.type = "gauge"
        metric.values = [mem]
        metric.dispatch()

    # File system usage
        try:
            fs_usage = (subprocess.check_output("export KUBECONFIG=/etc/kubernetes/admin.conf; kubectl exec -it %s -- du -s /run/kubevirt-private/vmi-disks/" %(pod_name), shell=True)).decode('utf8').strip().split('\t')[0]
            fs_usage = int(fs_usage) * 1024 # to bytes in binary
        except:
            fs_usage = 0
        value = fs_usage
        metric = collectd.Values()
        metric.plugin = PLUGIN_NAME
        metric.plugin_instance = "fs_usage"
        metric.type_instance = pod_name + '_VM'
        metric.type = "gauge"
        metric.values = [value]
        metric.dispatch()

    # VM CPU usage
        print_vm_stats_ifdb()

        stats_interval = 0
        found = False
        cpu_usage = (subprocess.check_output("export KUBECONFIG=/etc/kubernetes/admin.conf; kubectl get --raw /api/v1/nodes/%s/proxy/metrics/resource | grep pod_cpu_usage_seconds_total | grep %s | cut -d' ' -f2" %(vmhost, pod_name), shell=True)).decode('utf8').strip()
        cpu_usage_time = (subprocess.check_output("export KUBECONFIG=/etc/kubernetes/admin.conf; kubectl get --raw /api/v1/nodes/%s/proxy/metrics/resource | grep pod_cpu_usage_seconds_total | grep %s | cut -d' ' -f3" %(vmhost, pod_name), shell=True)).decode('utf8').strip()        
        #log(cpu_usage)
        #log(cpu_usage_time)

        for i in range(len(IfDB_vm_stats)):
            if IfDB_vm_stats[i].pod_vmname == pod_name:
                found = True
                break

        if not found:
            IfDB_vm_stats.append(vm_stats(pod_name, cpu_usage, cpu_usage_time))
            collectd.info("%s: %s %s %s %d %s %s" % (PLUGIN_NAME, "Adding entry (vm stats db):", pod_name, ", time:", int(cpu_usage_time), " cpu usage: ", cpu_usage))
        else:
            stats_interval = float(cpu_usage_time) - float(IfDB_vm_stats[i].last_time)
            #if stats_interval >= RATE_INTERVAL:
            if True:
                #log("time to get and send rates!!!!!")
                # it is time for sending rates
                metric = collectd.Values()
                metric.plugin = PLUGIN_NAME
                metric.plugin_instance = "cpu_usage"
                metric.type_instance = vmname
                metric.type = "gauge"
                # send the rate
                rate = (((float(cpu_usage) - float(IfDB_vm_stats[i].last_cpu_value))*100*1000)/(float(stats_interval)))
                # x100 - to get percent usage; x1000 - because cpu_usage is seconds, cpu_usage_time is miliseconds
                metric.values = [rate]
                metric.dispatch()
                # save current counters in database
                IfDB_vm_stats[i].last_time = cpu_usage_time
                IfDB_vm_stats[i].last_cpu_value = cpu_usage
                collectd.info("%s: %s %s %s %d %s %s/%f" % (PLUGIN_NAME, "Updating entry (vm stats db):", pod_name, ", time:", int(cpu_usage_time), " cpu_usage/rate:", cpu_usage, rate))

        print_vm_stats_ifdb()

        #cpu_usage = (subprocess.check_output("export KUBECONFIG=/etc/kubernetes/admin.conf; kubectl top pod | grep %s | tr -s ' '| cut -d' ' -f2" %(pod_name), shell=True)).decode('utf8').strip()
        #if cpu_usage.endswith("m"):
        #    cpu = int(cpu_usage.rstrip("m"))
        #else:
        #    cpu = 0

        #cpu_requests = (subprocess.check_output("export KUBECONFIG=/etc/kubernetes/admin.conf; kubectl describe pod %s | grep cpu | tr -s ' '| cut -d' ' -f3" %(pod_name), shell=True)).decode('utf8').strip()
        #if cpu_requests.endswith("m"):
        #    cpu_req = int(cpu_requests.rstrip("m"))
        #else:
        #    cpu_req = 200

        #cpu_usage = int((cpu/cpu_req) * 100)
        #log("cpu")
        #log(cpu)
        #log(cpu_req)
        #log(cpu_usage)

        ##metric = collectd.Values()
        ##metric.plugin = PLUGIN_NAME
        ##metric.plugin_instance = "cpu_usage"
        ##metric.type_instance = pod_name
        ##metric.type = "gauge"
        ##metric.values = [cpu_usage]
        ##metric.dispatch()

    #out = subprocess.check_output(['udevadm', 'info', '-q', 'path', '-p', '/sys/class/net/{}'.format(ifname) ])
    #if out.split(b'/')[2].decode() != 'virtual':
    #subprocess.check_output("echo %d > /sys/class/net/%s/device/sriov_numvfs || true" % (vf_count, ifname), shell=True)

def init():
    """
    This method has been registered as the init callback; this gives the plugin a way to do startup
    actions.  We'll just log a message.


    :return: None
    """

    log("Plugin %s initializing..." % PLUGIN_NAME)


def shutdown():
    """
    This method has been registered as the shutdown callback. this gives the plugin a way to clean
    up after itself before shutting down.  We'll just log a message.

    :return: None
    """

    log("Plugin %s shutting down..." % PLUGIN_NAME)


def write(values):
    """
    This method has been registered as the write callback. Let's count the number of datapoints
    and emit that as a metric.

    :param values: Values object for datapoint
    :return: None
    """

    #global DATAPOINT_COUNT
    #DATAPOINT_COUNT += len(values.values)


def flush(timeout, identifier):
    """
    This method has been registered as the flush callback.  Log the two params it is given.

    :param timeout: indicates that only data older than timeout seconds is to be flushed
    :param identifier: specifies which values are to be flushed
    :return: None
    """

    #log("Plugin %s flushing timeout %s and identifier %s" % PLUGIN_NAME, timeout, identifier)


def log_cb(severity, message):
    """
    This method has been registered as the log callback. Don't emit log messages from within this
    as you will cause a loop.

    :param severity: an integer and small for important messages and high for less important messages
    :param message: a string without a newline at the end
    :return: None
    """

    pass


def notification(notif):
    """
    This method has been regstered as the notification callback. Let's count the notifications
    we receive and emit that as a metric.

    :param notif: a Notification object.
    :return: None
    """

    #global NOTIFICATION_COUNT
    #NOTIFICATION_COUNT += 1


if __name__ != "__main__":
    # when running inside plugin register each callback
    collectd.register_config(config)
    collectd.register_read(read)
    collectd.register_init(init)
    collectd.register_shutdown(shutdown)
    collectd.register_write(write)
    collectd.register_flush(flush)
    collectd.register_log(log_cb)
    collectd.register_notification(notification)
else:
    # outside plugin just collect the info
    read()
    if len(sys.argv) < 2:
        while True:
            time.sleep(10)
            read()
