#!/usr/bin/python3
import os
import socket
import threading
import time
import sys
import queue

QUEUE_MAXELEM=512
work_queue = queue.Queue(maxsize=QUEUE_MAXELEM)
import vppapi_if.vppapi_if as vppif
from pyroute2 import IPRSocket
from systemd import journal

RTMGRP_LINK = 0x1
retcode = 0
vpp_json_dir = '/usr/share/vpp/api'
running = True
vppifs = dict()
collectd_path = '/tmp/collectd_sock'
event_stream = journal.stream(identifier='vpp_events')
vpp_died = False
def talk_to_collectd():
        global running
        global work_queue
        while running:
            try:
                (ifname, state) = work_queue.get(block=True, timeout=2)
                send_notification(ifname, state)
            except queue.Empty:
                pass
            except:
                pass


# Actually send the data
def send_notification(ifname, state):
    PLUGIN_NAME = 'vpp_events'
    message = 'link state of \\"%s\\" interface has been changed to \\"%s\\"' % (ifname, state)
    severity = "failure"
    if state == 'UP':
        severity = "okay"

    tm = time.time()
    
    try:
        with open('/config/collectd-hostname', 'r') as f:
            host = f.readline().strip()
    except:
        host=socket.gethostname()

    msg = "PUTNOTIF type=gauge type_instance=link_status plugin=vpp_events plugin_instance=%s host=%s severity=%s time=%s message=\"%s\"\n" % (
        ifname, host, str(severity), str(tm), message)
    # echo "PUTNOTIF type=gauge type_instance=link_status plugin=vpp_events plugin_instance=data1 host=pc5000 severity=okay time=$(date +%s) message=\"link state of \\\"data1\\\" interface has been changed to \\\"UP\\\"\"" | socat - UNIX-CLIENT:/tmp/collectd_sock

    if os.path.exists(collectd_path):
        try:
            cl = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)
            cl.connect(collectd_path)
            cl.sendall(msg.encode('utf-8'))
            cl.setblocking(0)
            time.sleep(0.01)
            ch = cl.recv(1)
            while ch != "\n":
                ch = cl.recv(1)

            cl.close()
        except Exception as ex:
            if "Resource temporarily unavailable" not in str(ex):
                print("exception on link change notif send %s" % str(ex), file=event_stream)
        # print ("exception 1 %s" % str(ex),file=event_stream)


def vpp_event_handler(msgname, result):
    global work_queue
    global vpp_died
    try:
        (id, client_index, pid, sw_if_index, admin_up_down, link_up_down, deleted) = result
        # refresh data for interface, as event has old state
        if sw_if_index:
            vppifs[sw_if_index] = vif.get_if_by_ifidx(sw_if_index)

        if sw_if_index in vppifs and 'ifname' in vppifs[sw_if_index]:
            name = vppifs[sw_if_index]['ifname']
            if not 'data' in name and not 'VirtualEthernet' in name:
                return
            dataname = name.replace("host-", "")
            if 'link_up_down' in vppifs[sw_if_index]:
                link_up_down = vppifs[sw_if_index]['link_up_down']
            if'admin_up_down' in vppifs[sw_if_index]:
                admin_up_down = vppifs[sw_if_index]['admin_up_down']
            if "VirtualEthernet" in name and not 'link_up_down' in vppifs[sw_if_index]:
                link_up_down = admin_up_down
            if link_up_down == 1:
                work_queue.put((dataname, "UP"), block=True, timeout=1)
                return
            if admin_up_down == 0 or (admin_up_down == 1 and link_up_down == 0):
                work_queue.put((dataname, "DOWN"), block=True, timeout=1)
    except Exception as ex:
        print("exception on link change processing %s" % str(ex), file=event_stream)
        if "It is likely that VPP died" in str(ex):
            vpp_died = True


# sw_interface_event(_0=83, client_index=0, pid=20874, sw_if_index=2, admin_up_down=0, link_up_down=0, deleted=0)

def vppevent_worker():
    global vif
    global running
    global work_queue
    global vpp_died
    vif.register_event_callback(vpp_event_handler)
    vif.want_interface_events(True)
    print("vpp_events vpp thread started", file=event_stream)
    while running:
        time.sleep(10)

        if vpp_died:
            vpp_died = False
            del vif
            time.sleep(3)
            vif = vppif.VppApiInterface('vpp-events' , rx_qlen=2048)
            print("vpp_events vpp api restarted", file=event_stream)
            vif.register_event_callback(vpp_event_handler)
            vif.want_interface_events(True)

def netlink_worker():
    global running
    global work_queue
    try:
        ip = IPRSocket()
        ip.bind()
        print("vpp_events netlink thread started", file=event_stream)
        while (running):
            msgs = ip.get()
            for msg in msgs:
                if 'event' in msg and 'attrs' in msg and msg['event'] == 'RTM_NEWLINK':
                    d = dict(msg['attrs'])
                    if 'IFLA_IFNAME' in d and 'IFLA_OPERSTATE' in d:
                        ifname = d['IFLA_IFNAME']
                        if 'wwan' in ifname:
                            continue
                        if 'DOWN' in d['IFLA_OPERSTATE']:
                            if 'flags' in msg and ((msg['flags'] & 0x10040) != 0):
                                continue
                            work_queue.put((ifname, "DOWN"), block=True, timeout=1)
                        else:
                            work_queue.put((ifname, "UP"), block=True, timeout=1)
    except Exception as ex:
        pass


if __name__ == '__main__':
    print("vpp_events is starting", file=event_stream)
    collectd_thr = threading.Thread(name='evt', target=talk_to_collectd)
    collectd_thr.start()
    vif = vppif.VppApiInterface('vpp-events' , rx_qlen=2048)
    nl_thr = threading.Thread(name='nl', target=netlink_worker)
    nl_thr.start()
    vpp_thr = threading.Thread(name='evt', target=vppevent_worker)
    vpp_thr.start()

    nl_thr.join()
    vpp_thr.join()
    collectd_thr.join()
    sys.exit(retcode)
