#!/bin/bash
exec 1> >(systemd-cat -p debug -t eth_up_down_vpp_process) 2>&1
set -o xtrace
set -o nounset
set -o pipefail
set -o errexit
interface=${1}
physical=0
self=${BASH_SOURCE[0]}

if grep -q data <<<"$interface"; then
  physical=1
  linkstat=$(sysrepocfg -X -x "/ucpe:config/interfaces:interface[name='${interface}']/link-state-propagation" -f json | grep link-state-propagation) || true
  if [ -n "$linkstat" ]; then
    if grep -q false <<<"$linkstat"; then
      tracestat=$(sysrepocfg -X -x "/ucpe:config/interfaces:interface[name='${interface}']/tracehost/trace-state-propagation" -f json | grep trace-state-propagation) || true
      if grep -q false <<<"$tracestat"; then
        echo "${interface} linkstat disabled"
        exit 0
      fi
    fi
  fi
elif grep -q lte <<<"$interface"; then
  physical=1
elif grep -q wifi <<<"$interface"; then
  physical=1
elif grep -q ppp <<<"$interface"; then
  physical=1
fi

for pid in $(pgrep -f "${self} ${interface}"); do
    if [ $pid != $$ ]; then
        process=$(ps -p ${pid} -o comm= || true)
        [[ "${process}" == "eth_up_down_vpp" ]] && kill $pid
    fi
done

state=${2}
#
if [[ ${physical} -eq 1 ]] ; then
  linkstat=$(sysrepocfg -X -x "/ucpe:config/interfaces:interface[name='${interface}']/link-state-propagation" -f json | grep link-state-propagation) || true
  if [ -n "$linkstat" ]; then
    if grep -q true <<<"$linkstat"; then
      if [ "${state,,}" == "up" ]; then
        sudo touch /config/_reboot/ifup/br-${interface}
      else
        sudo touch /config/_reboot/ifdown/br-${interface}
      fi
    fi
  fi
fi
#
exit 0 #mechanism on K8S is different
cd /run
{ for (( i = 0; i < 240; i++ )); do
    if [[ ! $(ls *.vnic) ]]; then
        sleep 1
        continue
    fi
    break
done; } 2>/dev/null
for file in $(ls *.vnic); do
  echo "File ${file} physical ${physical}"
  { for (( timeout = 0; timeout < 10; timeout++ )); do
      [ -s ${file} ] && break
      sleep 1
  done; } 2>/dev/null
  if grep -q lte <<<"$interface"; then
    if [ -z "$(grep "tap100" ${file})" ]; then
      echo "No ${interface} in ${file}"
      continue
    fi
  elif grep -q wifi <<<"$interface"; then
    if [ -z "$(grep "tap400" ${file})" ]; then
      echo "No ${interface} in ${file}"
      continue
    fi
  elif grep -q ppp <<<"$interface"; then
    if [ -z "$(grep "tap70" ${file})" ]; then
      echo "No ${interface} in ${file}"
      continue
    fi
  else
    if [ -z "$(grep ${interface} ${file})" ]; then
      echo "No ${interface} in ${file}"
      continue
    fi
  fi
  repeat=1
  max_repeat=60
  portname=$(grep port_name ${file} | cut -f2 -d':' | cut -f1 -d',' | tr -d "[:blank:]" | tr -d "'" ) || true
  while [[ ${physical} -eq 0  ]] && [[ -z "${portname}" ]] && [[ ${repeat} -le ${max_repeat} ]]; do
    sleep 1
    echo "Retry ${repeat} get port name "
    portname=$(grep port_name ${file} | cut -f2 -d':' | cut -f1 -d',' | tr -d "[:blank:]" | tr -d "'" ) || true
    repeat=$(( ${repeat} + 1 ))
  done
  if [ -z "${portname}" ]; then
    echo "NO port name FAIL, fail, ERROR, error"
    exit 0
  fi
  dev=$(grep dev= ${file} | sed 's/dev=//g' | tr -d '"')
  mac=$(grep mac= ${file} | sed 's/mac=//g' | tr -d '"')
  vmname=$(grep vmname= ${file} | sed 's/vmname=//g' | tr -d '"')
  echo "Found ${vmname} with ${dev} name ${portname}"
  echo "Physical ${physical} state ${state}"
  if [[ -n "${portname}" ]] && [[ ${physical} -eq 0 ]] && [[ "${state,,}" == "up" ]] ; then
    ovsdown=0
    linkstdisabled=0

    if [[ -f "/sys/class/net/${portname}/operstate" ]] ; then
      #this is virtio if, check kernel
      ovsstat=$(sudo cat /sys/class/net/${portname}/operstate) || true
    else
      #vpp
      ovsstat=$(sudo /usr/bin/vppctl show hardware-interfaces ${portname} | tr -d "\r" | head -5) || true
    fi

    if grep -q down <<<"$ovsstat"; then
      ovsdown=1
    fi
    linkstat=$(sysrepocfg -X -x "/ucpe:config/interfaces:interface[name='${portname}']/link-state-propagation" -f json | grep link-state-propagation) || true
    if [ -n "${linkstat}" ]; then
      if grep -q false <<<"$linkstat"; then
        linkstdisabled=1
      fi
    fi
    if [[ ${linkstdisabled} -eq 0  ]] && [[ ${ovsdown} -eq 1  ]] ; then
      virsh --connect=qemu:///system domif-setlink --domain ${vmname} --interface ${mac} --state down
    fi
  fi

  if [[ ${physical} -eq 1 ]] ; then
    virsh --connect=qemu:///system domif-setlink --domain ${vmname} --interface ${mac} --state ${state,,}
  fi
done
#
exit 0
